function [Source,Contribution,W] = Deflation(y,Param,FreqCycl)

% [Source,Contribution,W] = Deflation(x,Param,FreqCycl)
%
% Blind source separation by a deflation based method
% This function works for :
% - instantaneous or convolutive mixtures of sources
% - stationary or cyclostationary sources
% Run DemoNog to see usage examples and GenerateParamStruct to assist you
% generating a correct Param stucture array.
%
% Inputs:
% -------
% x = (Nx x Nsample) matrix containing the observations resulting from the
%   mixture (Nx sensors and Nsample samples)
% Param = structure array variables which controls the behaviour and the
%   different options of the function. Use the function GenerateParamStuct 
%   to assist you in setting the structure variable Param and to obtain 
%   more information.
% FreqCycl = (optionnal, for cyclo-stationnary sources only) array 
%   containing the cyclic frequencies >0. If you don't know what to do, we
%   recommend you not to pass a third argument. This input argument is used
%   only if Param.method = 'cyclostat'
%
% Outputs:
% -------
% Source = (Ns x Nsample) array containing the Ns separated signals. 
%   In case of i.i.d. sources, the Ns separated signals correspond to the
%   sources (up to permutation, scaling factor and delay); in the case of
%   non i.i.d. sources, the separated signals correspond to the estimation
%   of filtered versions of the Ns sources.
% Contribution = (Ns x Nx x Nsample') array containing the contribution 
%   of each source signal on each sensor. (Nsample'<=Nsample depending
%   whether the signals have been truncated during the separation process)
% W = MIMO-FIR separator coefficient (3D array, which should be used as an
%   input argument for the function MIMOfilt)
%
% -------
% Author: Pierre Jallon / Marc Castella
% Created: 22/04/2005
% Last modified: 20/08/2009
%               05/05/2010
% Contact: marc.castella@int-evry.fr

% TO DO: permettre de passer l'option 'choix_mu' 
% -> 10/08/2008: choix_mu = Param.GradientStep;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Parameters test and initialization :
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

global TLBXPATH; if isempty(TLBXPATH), TLBXPATH=pwd; end;
addpath(sprintf('%s/CycloStat/',TLBXPATH));
addpath(sprintf('%s/CycloStat/SourceExtraction/',TLBXPATH));
addpath(sprintf('%s/Stationary/KurtosisContrast/',TLBXPATH));
addpath(sprintf('%s/Stationary/QuadraticContrast/',TLBXPATH));
addpath(sprintf('%s/Stationary/CubicContrast/',TLBXPATH));

if (nargin==0)
    disp('You need to pass the mixture as a parameter');
    exit;    
end

if (nargin==1)
    Param = GenerateParameters;
    FreqCycl = [];
end

if (nargin==2)
    FreqCycl = [];
end


if (size(y,1)>size(y,2))
    y = y.';    
end

StillSource = true;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Deflation procedure :
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

MixturePower = sum(sum(abs(y).^2));
iDetectedSource = 1;

% Variables for gradient algorithm
%%%Direction = 0;
%%%OldGradNorm = 1;

D = Param.LFilter; Nobserv = size(y,1);

y_trunc = y;
Ns_trunc = length(y);

Contribution = zeros(1,Nobserv,Ns_trunc);

while(StillSource)    
    
%fprintf('Extracting source %d...\n',iDetectedSource);
switch Param.Method,
    case 'cyclostat',
        g = GradientCyclo(y_trunc,D,Param.NbIterationMax,Param.Seuil,FreqCycl);
        W(iDetectedSource,:,:) = reshape(g.',1,Nobserv,D);
    case 'kurtosisRealValued',
        %%% ne marche que pour des sources reelles, similaire a 'kurtosis'
        choix_mu = Param.GradientStep;
        f_ini = zeros(1,Nobserv*D); f_ini(round(Nobserv*D/2)) = 1;
        [W(iDetectedSource,:,:)] = GradientKurtR(Param.Seuil,Param.NbIterationMax,y_trunc,f_ini,choix_mu);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
    case 'kurtosis',
        choix_mu = Param.GradientStep;
        f_ini = zeros(1,Nobserv*D); f_ini(round(Nobserv*D/2)) = 1;
        [W(iDetectedSource,:,:)] = GradientKurtC(Param.Seuil,Param.NbIterationMax,y_trunc,f_ini,choix_mu);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
    case 'kurtosis_CPM',
        %%% le critere est -Kurtosis (sans abs, ni carre)
        choix_mu = Param.GradientStep;
        f_ini = zeros(1,Nobserv*D); f_ini(round(Nobserv*D/2)) = 1;
        [W(iDetectedSource,:,:)] = GradientKurtC_CPM(Param.Seuil,Param.NbIterationMax,y_trunc,f_ini,choix_mu);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
    case {'quadSVD','quadratic'}, %backward compatibility. 'quadratic' was used in former version
        L = Param.Quadratic.L;
        warning('Assumed length of the mixing filter is %d.\n\t Poor performance may be observed with a wrong value.\n\t Try the method quadGrad for improved performance.',L);
        Y = [];
        for d = 1:D,
            Y = [ y_trunc(:,d:end+d-D); Y];
        end;
        w = zeros(1,Nobserv*D); w(1) = 1;
        z = w*Y;
        for iterptfix = 1:Param.Quadratic.Nifix,
            [R,C] = makeRC(y_trunc,z,D);
            w = reswCw(C,R,(iDetectedSource-1)*(D+L-1),min(Nobserv*D,Param.NsourcesEffectiv*(D+L-1)));
            z = w*Y;
        end;
        W(iDetectedSource,:,:) = reshape(w,1,Nobserv,D);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
    case {'refSVD','reference'}, %backward compatibility. 'reference' was used previously
        L = Param.Quadratic.L;
        warning('Assumed length of the mixing filter is %d.\n\t Poor performance may be observed with a wrong value.\n\t The method refGrad is currently beeing developped for improved performance.',L);
        Y = [];
        for d = 1:D,
            Y = [ y_trunc(:,d:end+d-D); Y];
        end;
        z = Param.Reference(iDetectedSource,1:length(Y));
        for iterptfix = 1:1;%Param.Quadratic.Nifix,
            [R,C] = makeRC(y_trunc,z,D);
            w = reswCw(C,R,(iDetectedSource-1)*(D+L-1),min(Nobserv*D,Param.NsourcesEffectiv*(D+L-1)));
            z = w*Y;
        end;
        W(iDetectedSource,:,:) = reshape(w,1,Nobserv,D);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
    case {'refGrad'},
        error('Not implemented yet. Please use refSVD or quadQrad or quadSVD.');
    case {'quadSVDUnknownRank','quadraticUnknownRank'},
        warning('Poor performance may be observed.\n\t Try the methods quadSVD or quadGrad for improved performance.',[]);
        % pas droit a: L = Param.Quadratic.L;
        Y = [];
        for d = 1:D,
            Y = [ y_trunc(:,d:end+d-D); Y];
        end;
        w = zeros(1,Nobserv*D); w(1) = 1;
        z = w*Y;
        for iterptfix = 1:Param.Quadratic.Nifix,
            [R,C] = makeRC(y_trunc,z,D);
            w = reswCwUnknownRank(C,R);%,(iDetectedSource-1)*(D+L-1));%,min(Nobserv*D,Param.NsourcesEffectiv*(D+L-1)));
            z = w*Y;
        end;
        W(iDetectedSource,:,:) = reshape(w,1,Nobserv,D);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
    case 'quadraticGradR',
        warning('Method suitable only for specific sources.\n\tContrast used: Cum{y(n),y(n),r(n),r(n)} \t (No abs)',[]);
        Y = [];
        for d = 1:D,
            Y = [ y_trunc(:,d:end+d-D); Y];
        end;
        epsilon = Param.Seuil; Nmax = Param.NbIterationMax;
        choix_mu = Param.GradientStep;
        f_ini = zeros(1,Nobserv*D); f_ini(round(Nobserv*D/2)) = 1;
        w = f_ini; z=w*Y;
        for iterptfix = 1:Param.Quadratic.Nifix,
            [w] = GradientQuadR(epsilon,Nmax,y_trunc,z,f_ini,choix_mu);
            z = w*Y;
        end;
        W(iDetectedSource,:,:) = reshape(w,1,Nobserv,D);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
    case {'quadGradRealValued','quadraticGradR2'},
        Y = [];
        for d = 1:D,
            Y = [ y_trunc(:,d:end+d-D); Y];
        end;
        epsilon = Param.Seuil; Nmax = Param.NbIterationMax;
        choix_mu = Param.GradientStep;
        f_ini = zeros(1,Nobserv*D); f_ini(round(Nobserv*D/2)) = 1;
        w = f_ini; z=w*Y;
        for iterptfix = 1:Param.Quadratic.Nifix,
            [w] = GradientQuadR2(epsilon,Nmax,y_trunc,z,f_ini,choix_mu);
            z = w*Y;
        end;
        W(iDetectedSource,:,:) = reshape(w,1,Nobserv,D);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
    case 'quadraticGradC',
        warning('Method suitable only for specific sources.\n\tContrast used: Cum{y(n),y(n)*,r(n),r(n)*} \t (No modulus)',[]);
        Y = [];
        for d = 1:D,
            Y = [ y_trunc(:,d:end+d-D); Y];
        end;
        epsilon = Param.Seuil; Nmax = Param.NbIterationMax;
        choix_mu = Param.GradientStep;
        f_ini = zeros(1,Nobserv*D); f_ini(round(Nobserv*D/2)) = 1;
        w = f_ini; z=w*Y;
        for iterptfix = 1:Param.Quadratic.Nifix,
            [w] = GradientQuadC(epsilon,Nmax,y_trunc,z,f_ini,choix_mu);
            z = w*Y;
        end;
        W(iDetectedSource,:,:) = reshape(w,1,Nobserv,D);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
    case {'quadGrad','quadraticGradC2'},
        Y = [];
        for d = 1:D,
            Y = [ y_trunc(:,d:end+d-D); Y];
        end;
        epsilon = Param.Seuil; Nmax = Param.NbIterationMax;
        choix_mu = Param.GradientStep;
        f_ini = zeros(1,Nobserv*D); f_ini(round(Nobserv*D/2)) = 1;
        w = f_ini; z=w*Y;
        for iterptfix = 1:Param.Quadratic.Nifix,
            [w] = GradientQuadC2(epsilon,Nmax,y_trunc,z,f_ini,choix_mu);
            z = w*Y;
        end;
        W(iDetectedSource,:,:) = reshape(w,1,Nobserv,D);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
        %% je voulais surement developper quadraticImprGrad...
%     case 'quadraticGradDbleIni',
%         Y = [];
%         for d = 1:D,
%             Y = [ y_trunc(:,d:end+d-D); Y];
%         end;
%         epsilon = Param.Seuil; Nmax = Param.NbIterationMax;
%         choix_mu = Param.GradientStep;
%         w = zeros(1,Nobserv*D); w(1) = 1;
%         z = w*Y;
%         f_ini = zeros(1,Nobserv*D); f_ini(round(Nobserv*D/2)) = 1;
% %        w = f_ini; z=w*Y;
%         [w] = GradientQuadC2(epsilon,Nmax,y_trunc,z,f_ini,choix_mu);
%         W(iDetectedSource,:,:) = reshape(w,1,Nobserv,D);
%         g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
    case 'quadImprGradOld',
        warning('quadImprGradOld is now obsolete. quadImprGrad should give the same result, but is computationaly more efficient.');
        Y = [];
        for d = 1:D,
            Y = [ y_trunc(:,d:end+d-D); Y];
        end;
        epsilon = Param.Seuil; Nmax = Param.NbIterationMax;
        choix_mu = Param.GradientStep;
        f_ini = zeros(1,Nobserv*D); f_ini(round(Nobserv*D/2)) = 1;
        w = f_ini; z=w*Y;
        for iterptfix = 1:Param.Quadratic.Nifix,
            [w] = ImprGradientQuadC2OLD(epsilon,Nmax,y_trunc,z,w,choix_mu);
            z = w*Y;
        end;
        W(iDetectedSource,:,:) = reshape(w,1,Nobserv,D);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
    case 'quadImprGrad',
        epsilon = Param.Seuil; Nmax = Param.NbIterationMax;
        choix_mu = Param.GradientStep;
        f_ini = zeros(1,Nobserv*D); f_ini(round(Nobserv*D/2)) = 1;
        Nifix = Param.Quadratic.Nifix;
        [w] = ImprGradientQuadC2(epsilon,Nmax,y_trunc,f_ini,Nifix,choix_mu);
        W(iDetectedSource,:,:) = reshape(w,1,Nobserv,D);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
    case 'quadImprGrad_CPM',
        epsilon = Param.Seuil; Nmax = Param.NbIterationMax;
        choix_mu = Param.GradientStep;
        f_ini = zeros(1,Nobserv*D); f_ini(round(Nobserv*D/2)) = 1;
        Nifix = Param.Quadratic.Nifix;
        [w] = ImprGradientQuadC(epsilon,Nmax,y_trunc,f_ini,Nifix,choix_mu);
        W(iDetectedSource,:,:) = reshape(w,1,Nobserv,D);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).'; 
    case {'cubiHOPM'},
        L = Param.Cubic.L;
        algo.loop=Param.NbIterationMax; algo.epsilon=Param.Seuil; algo.stop=Param.USeuil; 
        
        algo.cumstar=[0 1 1 0];  algo.order=3;
        warning('Assumed length of the mixing filter is %d.\n\t Poor performance may be observed with a wrong value.\n',L);
        Y = [];
        for d = 1:D,
            Y = [ y_trunc(:,d:end+d-D); Y];
        end;
        w = zeros(1,Nobserv*D); w(1) = 1;
        z = w*Y;
        for iterptfix = 1:Param.Cubic.Nifix,
            [R,C] = Cumulant(algo.cumstar,algo.order,y_trunc,Y,z,D);
                        
            w = resCwww(algo,C,R,(iDetectedSource-1)*(D+L-1),min(Nobserv*D,Param.NsourcesEffectiv*(D+L-1)));
            z = w*Y; 
        end;
        W(iDetectedSource,:,:) = reshape(w,1,Nobserv,D);
        g = reshape(W(iDetectedSource,:,:),Nobserv,D).';
        

end;

    if strcmp(Param.Method,'cyclostat'), 
    % pour raisons compatibilite avec Demog, eviter de tronquer qd on fait
    % appel aux fonctions made in Jallon
        fprintf('Cancelling its contribution ...\n');
        r = ComputeSignal(g,y_trunc);
        Source(iDetectedSource,1:length(r)) = r;
        %%%PAS TRONCATURE%%% Ns_trunc = Ns_trunc-D+1;
        % Cancelling the extracted source contribution :
        [ContributionEst,FiltreExtTemp] = SubtractSource(y_trunc,r,Param.LSubtractFiltreAC,Param.LSubtractFiltreC);
        % Save Extracted Source:
        %%%PAS TRONCATURE%%% Contribution = Contribution(:,:,1:Ns_trunc);
        Contribution(iDetectedSource,:,:) = permute(ContributionEst,[3 1 2]);
        % Updating mixture
        y_trunc = y_trunc-ContributionEst;
    else
        %fprintf('Cancelling its contribution ...\n');
        r = ComputeSignal(g,y_trunc);
        Source(iDetectedSource,1:length(r)-D+1) = r(D:end); %bug corrige 22/08/2008
        Ns_trunc = Ns_trunc-D+1; %%%TRONCATURE%%%
        % Cancelling the extracted source contribution :
        [ContributionEst,FiltreExtTemp] = SubtractSource(y_trunc(:,1:Ns_trunc),r(D:end),Param.LSubtractFiltreAC,Param.LSubtractFiltreC);
        % Save Extracted Source:
        Contribution = Contribution(:,:,1:Ns_trunc); %%%TRONCATURE%%%
        Contribution(iDetectedSource,:,:) = permute(ContributionEst,[3 1 2]);
        % Updating mixture
        y_trunc = y_trunc(:,1:Ns_trunc)-ContributionEst;
    end;
    
    % Still source ?
    NewMixturePower = sum(sum(abs(y_trunc).^2));
        
    if (NewMixturePower/MixturePower<Param.SeuilStopAlgo) 
        StillSource = false;
    elseif (iDetectedSource==(Param.Nsources))
        StillSource = false;
    elseif (iDetectedSource == size(y_trunc,1)),
        StillSource = false;
    else
        iDetectedSource = iDetectedSource+1;
    end
end

% identification filtre MIMO separateur
if ~strcmp(Param.Method,'cyclostat'),
    Y = [];
    for d = 1:D,
        Y = [ y(:,d:end+d-D); Y];
    end;
    Y = Y(:,1:Ns_trunc);
    Wtmp = Source(:,1:Ns_trunc)/Y; %ancienne version pose pb
%    Wtmp = Source(:,end-Ns_trunc+1:end)/Y; %nouvelle version pose pb aussi
    W=reshape(Wtmp,size(Source,1),Nobserv,D);
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Cleaning : 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% rmpath(sprintf('%s/CycloStat/',TLBXPATH));
% rmpath(sprintf('%s/CycloStat/SourceExtraction/',TLBXPATH));
% rmpath(sprintf('%s/Stationary/KurtosisContrast/',TLBXPATH));
% rmpath(sprintf('%s/Stationary/QuadraticContrast/',TLBXPATH));
