% This script should reproduce the results given in the following 
% published paper (minor changes are possible).
%
% [1] Marc Castella and Eric Moreau, A New Method for Kurtosis Maximization
%   and Source Separation. Proc. of ICASSP 2010, pp.2670-2673, 14-19 March,
%   Dallas, TX, USA.
%
% -------
% Author: Marc Castella
% Created: ??/2009
% Last modified: 05/03/2010
% Contact: marc.castella@it-sudparis.eu

%%
clear;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Uncomment the line according to the expected result
%SimulationTableNumber = 1; % reproduces Table 1 in [1]
SimulationTableNumber = 2; % reproduces Table 2 in [1]

%% Choose the number of samples and number of Monte-Carlo runs
Nsample = 5000;
NTEST = 1000;

%% define your toolbox path
cd ..; 
global TLBXPATH; TLBXPATH = pwd;

%% set warning on or off
warning('on');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath(sprintf('%s/Stationary/SourceGen_Filt/',TLBXPATH));
addpath(sprintf('%s/Stationary/',TLBXPATH));
addpath(TLBXPATH);

%---generate a mixture-----------------------------------------------------
Nsource = 3;        %number of sources
Nobserv = Nsource+1;%number of sensors
L = 3;              %length of the mixing filter
D=max(1,Nsource*(L-1));    %length of the separating filter (set to the minimum 
                    %value so as to ensure invertibility of the mixture)
decalmax = (L+D-2); % correction bug 29/08/2008: a confirmer...
         
%---set a Param structure for use with Deflation.m-------------------------
MyParameters.LFilter            = D;      %length of the mixing filter
MyParameters.Seuil              = 0;      %threshold value for convergence 
                                          %of the gradient. If the norm
                                          %of the parameters vector change
                                          %is less than this value, no other 
                                          %gradient update is peformed.
switch SimulationTableNumber,
    case 1,
        MyParameters.Nsources = 1;        %number of sources to be extracted
    case 2,
        MyParameters.Nsources = Nsource;
end;
MyParameters.SeuilStopAlgo      = 0;      %threshold value on the power of 
                                          %the mixture to determine whether
                                          %there are other sources or not. 
                                          %Under this value, it is 
                                          %considered that there is no more
                                          %source to be extracted and no
                                          %other deflation is performed. 

MyParameters.LSubtractFiltreAC  = L-1;    %length of the anti-causal part
                                          %of the subtracting filter in 
                                          %the deflation step
MyParameters.LSubtractFiltreC   = D+L-2;  %length of the causal part of  
                                          %the subtracting filter in the 
                                          %deflation step
%---additionnal parameters required for 'quadratic' contrasts--------------
MyParameters.NsourcesEffectiv = Nsource;%number of sources effectively 
                                        %present in the mixture (assumed to 
                                        %be known)
MyParameters.Quadratic.L = L;    %length of the mixing filter (assumed to 
                                 %be known)
MyParameters.GradientStep = 'optim';


for ntest = 1:NTEST,
 fprintf('---ntest = %d ----------------------------------\n',ntest);
 switch SimulationTableNumber,
     case 1,
         s = generSources(Nsource,Nsample,6);                   %random generation of the sources
         M = randn(Nobserv,Nsource,L)+i*randn(Nobserv,Nsource,L);       %random generation of the mixing filter
     case 2,
         s(1,:) = generSources(1,Nsample,9);
         s(2:Nsource,:) = generSources(Nsource-1,Nsample,2);   %random generation of the sources
         M = randn(Nobserv,Nsource,L);                         %random generation of the mixing filter
 end;

x = MIMOfilt(M,s);                  %convolutive mixing of the sources

MyParameters.Method = 'kurtosis';
MyParameters.NbIterationMax     = 1000;   %maximum gradient iterations
MyParameters.Quadratic.Nifix = 1;%number of "fixed point" iterations
[y,Contribution,g] = Deflation(x,MyParameters,[]);
[MSE.kurtosis(ntest,:),news,newy] = elimamb_g3(s,y,decalmax); %compute the mean square error

MyParameters.Method = 'quadGrad';
MyParameters.NbIterationMax     = 1000;   %maximum gradient iterations
MyParameters.Quadratic.Nifix = 1;%number of "fixed point" iterations
[y,Contribution,g] = Deflation(x,MyParameters,[]);
[MSE.quadGrad(ntest,:),news,newy] = elimamb_g3(s,y,decalmax); %compute the mean square error


MyParameters.Method = 'quadImprGrad';
MyParameters.NbIterationMax     = 1;   %maximum gradient iterations
MyParameters.Quadratic.Nifix = 1000;%number of "fixed point" iterations
[y,Contribution,g] = Deflation(x,MyParameters,[]);
[MSE.quadImprGrad(ntest,:),news,newy] = elimamb_g3(s,y,decalmax); %compute the mean square error
                        
end; %for ntest                                 

clear Contribution M g news newy s t0 x y

rmpath(sprintf('%s/Stationary/SourceGen_Filt/',TLBXPATH));
rmpath(sprintf('%s/Stationary/',TLBXPATH));
rmpath(TLBXPATH);

cd PublishedResults;

%% display the results
Resu = ['kurtosis     : ';
        'ref. contrast: '; 
        'Alg.1        : '];

disp('mean values (averaged over all realizations)');
disp( [Resu, num2str( [mean(MSE.kurtosis); mean(MSE.quadGrad); mean(MSE.quadImprGrad)] ) ]) ;
disp('median values (averaged over all realizations)');
disp( [Resu, num2str( [median(MSE.kurtosis); median(MSE.quadGrad); median(MSE.quadImprGrad)] ) ]) ;
disp('---END-----------------------------------------');
