% This script should reproduce the results given in the following 
% published paper (minor changes are possible).
%
% [1] Marc Castella and Eric Moreau, New kurtosis optimization schemes 
%   for MISO equalization. Accepted (Nov. 2011) for publication in 
%   IEEE Trans. Signal Processing.
%
% Executing this script with adequate parameters should reproduce the
% results in Tables I, III, IV and V. Minor changes in the MSE are 
% possible and changes in the execution time are very likely. The other 
% results in [1] have been obtained with the same programs.
%
% -------
% Author: Marc Castella
% Created: ??
% Last modified: 21/11/2011
% Contact: marc.castella@it-sudparis.eu


%%
clear;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Choose the number of samples and number of Monte-Carlo runs
Nsample = 5000;
NTEST = 1000;

%% choose the separation method and parameters kmax,lmax
MyParameters.Method = 'quadImprGrad';   %Alg.2 as described in [1]
% MyParameters.Method = 'kurtosis';       %gradient optimization of kurtosis
    kmax = 5;
    lmax = 200;

if strcmp(MyParameters.Method,'kurtosis'),
    lmax = max(kmax,lmax);  % for method 'kurtosis', number of iterations 
                            % given by lmax, kmax is ignored
end;

%% define your toolbox path
cd ..; 
global TLBXPATH; TLBXPATH = pwd;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath(sprintf('%s/Stationary/SourceGen_Filt/',TLBXPATH));
addpath(sprintf('%s/Stationary/',TLBXPATH));
addpath(TLBXPATH);

%---generate a mixture-----------------------------------------------------
Nsource = 3;        %number of sources
Nobserv = Nsource+1;%number of sensors
L = 3;              %length of the mixing filter
D=max(1,Nsource*(L-1));    %length of the separating filter (set to the minimum 
                    %value so as to ensure invertibility of the mixture)
decalmax = (L+D-2); % correction bug 29/08/2008: a confirmer...
         
%---set a Param structure for use with Deflation.m-------------------------
MyParameters.LFilter            = D;      %length of the mixing filter
MyParameters.Seuil              = 0;      %threshold value for convergence 
                                          %of the gradient. If the norm
                                          %of the parameters vector change
                                          %is less than this value, no other 
                                          %gradient update is peformed.

MyParameters.Nsources           = 1;%Nsource;%number of sources to be extracted
MyParameters.SeuilStopAlgo      = 0;      %threshold value on the power of 
                                          %the mixture to determine whether
                                          %there are other sources or not. 
                                          %Under this value, it is 
                                          %considered that there is no more
                                          %source to be extracted and no
                                          %other deflation is performed. 

MyParameters.LSubtractFiltreAC  = L-1;    %length of the anti-causal part
                                          %of the subtracting filter in 
                                          %the deflation step
MyParameters.LSubtractFiltreC   = D+L-2;  %length of the causal part of  
                                          %the subtracting filter in the 
                                          %deflation step
MyParameters.NbIterationMax =  lmax;
MyParameters.GradientStep = 'optim';
%---additionnal parameters required for 'quadratic' contrasts--------------
MyParameters.NsourcesEffectiv = Nsource;%number of sources effectively 
                                        %present in the mixture (assumed to 
                                        %be known)
MyParameters.Quadratic.L = L;    %length of the mixing filter (assumed to 
                                 %be known)
MyParameters.Quadratic.Nifix = kmax;      %number of "fixed point" iterations


totalTime = 0;

for ntest = 1:NTEST,
 fprintf('---ntest = %d ----------------------------------\n',ntest);
s = generSources(Nsource,Nsample,6);%random generation of the sources
M = randn(Nobserv,Nsource,L)+i*randn(Nobserv,Nsource,L);       %random generation of the mixing filter
x = MIMOfilt(M,s);                  %convolutive mixing of the sources

%% -------
t0=clock;
[y,Contribution,g] = Deflation(x,MyParameters,[]);
totalTime = totalTime + etime(clock,t0);
[MSE(ntest,:),news,newy] = elimamb_g3(s,y,decalmax); %compute the mean square error

                       
end; %for ntest                                 

clear Contribution M g news newy s t0 x y;


rmpath(sprintf('%s/Stationary/SourceGen_Filt/',TLBXPATH));
rmpath(sprintf('%s/Stationary/',TLBXPATH));
rmpath(TLBXPATH);

cd PublishedResults;

%% display the results
disp('mean value of MSE (averaged over all realizations)');
disp(mean(MSE));
disp('median value of MSE (averaged over all realizations)');
disp(median(MSE));
disp('average execution time in s (averaged over all realizations)');
disp(totalTime/ntest);
disp('---END-----------------------------------------');

