% This script should reproduce the results given in the following 
% published papers (minor changes are possible).
%
% [1] Marc Castella and Eric Moreau, Generalized Identifiability Conditions for 
%   Blind Convolutive MIMO Separation. IEEE Trans. Signal Processing, 
%   Vol. 57, Issue 7, Jul. 2009, pp.2846-2852.
%
% [2] Marc Castella and Eric Moreau, Reference Based Contrast Functions in a 
%   Semi-Blind Context.  Proc. of Eighth Int. Conf. ICA 2009, LNCS 
%   Vol.5441, pp.9-16, 19-18 March 2009 2007, Paraty-RJ, Brazil.
%
% -------
% Author: Marc Castella
% Created: ??/2008
% Last modified: 30/06/2009
% Contact: marc.castella@it-sudparis.eu

%%
clear;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Uncomment the line according to the expected result
% choixSources = 1 % i.i.d. binary +-1 with resp. probability 1/2 (BPSK)
choixSources = 2; % i.i.d. uniformly distributed
% choixSources = 6 % i.i.d. complex-valued QAM4
% choixSources = 7 % i.i.d. complex-valued QAM16

%% Choose the number of samples and number of Monte-Carlo runs
Nsample = 1000; % number of samples
NTEST = 1000; %number of Monte-Carlo realizations (1000 in [1] and [2])

%% Set the following parameters
Nsource = 3;        %number of sources (Nsource = 3 in [2] or 3/4 in [1])

%% Define a structure with following elements depending on your choice:
refer.info = 'phase'; refer.filtered = false; refer.param = 1;
%%% refer.info = 'phase' : if reference contains phase information (eq.
%%%                       (14) in [2]
%%%                 = 'modulus' if reference contains modulus information (see 
%%%                       eq. (16) in [2]
%%% refer.filteredRed = false : if reference given by eq. (14) or (16) in
%%%                           [2]
%%%                      = true : if reference given by eq. (14) or (16) +
%%%                           filtering (see [2])
%%% refer.param = parameter p or theta (see [2] §5.1)

%% define your toolbox path
cd ..; 
global TLBXPATH; TLBXPATH = pwd;
% TLBXPATH = '/home/castella/Rech/ProgBSSTlbx/DevelNewToolbox/';

%% set warning on or off
warning('off');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath(sprintf('%s/Stationary/SourceGen_Filt/',TLBXPATH));
addpath(sprintf('%s/Stationary/',TLBXPATH));
addpath(TLBXPATH);

%--- parameters (set to the values in [1],[2])-----------------------------
L = 3;              %length of the mixing filter
Nobserv = Nsource+1;%number of sensors
D=Nsource*(L-1);    %length of the separating filter (set to the minimum 
                    %value so as to ensure invertibility of the mixture)
decalmax = 2*(L+D-2);
%choixSources = 2;

%---set a Param structure for use with Deflation.m-------------------------
MyParameters.LFilter            = D;      %length of the mixing filter
MyParameters.Seuil              = 0;      %threshold value for convergence 
                                          %of the gradient. If the norm
                                          %of the parameters vector change
                                          %is less than this value, no other 
                                          %gradient update is peformed.
MyParameters.NbIterationMax     = 1000;   %maximum gradient iterations
MyParameters.Nsources           = 1;%Nsource;%number of sources to be extracted
MyParameters.SeuilStopAlgo      = 0;      %threshold value on the power of 
                                          %the mixture to determine whether
                                          %there are other sources or not. 
                                          %Under this value, it is 
                                          %considered that there is no more
                                          %source to be extracted and no
                                          %other deflation is performed. 

MyParameters.LSubtractFiltreAC  = L-1;    %length of the anti-causal part
                                          %of the subtracting filter in 
                                          %the deflation step
MyParameters.LSubtractFiltreC   = D+L-2;  %length of the causal part of  
                                          %the subtracting filter in the 
                                          %deflation step
%---additionnal parameters required for 'quadratic' contrasts--------------
MyParameters.NsourcesEffectiv = Nsource;%number of sources effectively 
                                        %present in the mixture (assumed to 
                                        %be known)
MyParameters.Quadratic.L = L;    %length of the mixing filter (assumed to 
                                 %be known)
MyParameters.Quadratic.Nifix = 1;%number of "fixed point" iterations (see 
                                 %the method descriptions for more details)

MyParameters.Method = 'reference';%chosen separation method


for ntest = 1:NTEST,
    
fprintf('---ntest = %d ----------------------------------\n',ntest);

s = generSources(Nsource,Nsample,choixSources);%random generation of the sources
M = randn(Nobserv,Nsource,L);       %random generation of the mixing filter
x = MIMOfilt(M,s);                  %convolutive mixing of the sources


if (choixSources==1)||(choixSources==2),
    % real valued case
    switch refer.info,
        case 'phase',
            MyParameters.Reference = sign(s).*( 2*(rand(size(s))<refer.param) -1 );
        case 'modulus',
            MyParameters.Reference = (sign(randn(size(s)))) .* abs(s);
    end;
elseif (choixSources==6)||(choixSources==7),
    % complex valued case
    switch refer.info,
        case 'phase',
            MyParameters.Reference = sign(s).*( exp( i*refer.param*(2*rand(size(s))-1) ) );
        case 'modulus',
            MyParameters.Reference = exp(i*2*pi*randn(size(s))) .* abs(s);
    end;
end;
if (refer.filtered == true),
    for kk = 1:3,
        Htmp(:,:,kk)= diag(randn(1,min(size(s))));
    end;
    MyParameters.Reference = MIMOfilt(Htmp,MyParameters.Reference);
end;
    


[y,Contribution,g] = Deflation(x,MyParameters,[]);
[MSE(ntest,:),news,newy] = elimamb_g3(s,y,decalmax);

end; %for ntest

rmpath(sprintf('%s/Stationary/SourceGen_Filt/',TLBXPATH));
rmpath(sprintf('%s/Stationary/',TLBXPATH));
rmpath(TLBXPATH);

cd PublishedResults;

%% display the results
disp('Mean value of the MSE');
disp(mean(MSE));
disp('---END-----------------------------------------');