function [w] = resCwww(algo,C,R,dimReduc,rk);

% w = resCwww(algo,C,R,dimReduc,rk);
%
% Optimization of a reference-based cubic contrast function 
% Source_Extraction performs the optimization of the cubic contrast described in
% [1] (see eq. (37))
%
% Inputs:
% -------
% algo = parameter for the hopm algorithm, stopping criterion and epsilon
% C = cumulant tensor (given by the program Cumulant.m)
% R = covariance matrix (given by the program Cumulant.m)
% rk = rank of the matrix R (if known, otherwise assumed value) for the 
%   first step of the separating procedure (no previous deflation step). 
%   This corresponds to the value r_0 in the global procedure described 
%   in [2].
% dimReduc = value of the rank deficiency due to previous deflation
%   procedures. dimReduc = r_0 - r_i at the i-th source extraction (see the
%   description of the global separation procedure in [2]).
% 
% Output: 
% -------
% w = (1 x Nx.D) row vector containing the Nx.D coefficients of the
%   row separating filter where Nx is the number of observation signals and
%   D the length of the separating filter
%
% -------
% Author: Rémi Dubroca
% Created: 06/2010
% Last modified: 01/07/2010
% Contact: dubroca@univ-tln.fr
%
% Ref:
% [1] A General Algebraic Algorithm for Blind Extraction
% of one Source in a MIMO Convolutive Mixture / R. Dubroca, C. De Luigi, M. Castella and E. Moreau
% IEEE Trans. on Signal Processing 58(5), May 2010
% [2] Quadratic Higher-Order Criteria for Iterative Blind Separation of a 
% MIMO Convolutive Mixture of Sources / M. Castella, S. Rhioui, E. Moreau 
% and J.-C. Pesquet, IEEE Trans. on Signal Processing 55(1), January 2007


prec=0; loopend=0; 

[U,S,V] = svd(R);
rg = rk-dimReduc;

%if rank(R)~=size(R,1), disp('Warning: R non de rang plein'); end;
%rg = size(R,1)-dimReduc;
if rank(R)<rg, disp('Warning: rank of R < assumed rank rk-dimReduc'); %k=diag(S).', 
    rg = rank(R); 
end;

Q = ( V(:,1:rg)*diag(1./sqrt(diag(S(1:rg,1:rg)))) ).';
P = diag(sqrt(diag(S(1:rg,1:rg))))*V(:,1:rg)'; % P=S^(1/2)*V^{H}

vecD(1:length(diag(S)),1)=diag(S);
cumstar=algo.cumstar;
Ct=C;
for i=1:ndims(C)
    Qe=conjcumproj(Q,cumstar(i));

    [Cb] = n_product(Ct,i,Qe);
    Ct=Cb;
end

[TxU,loopend] = hopm(Cb,algo.loop,algo.epsilon,algo.stop);
w = TxU(:,1);

wc=conj(w'*Q);
w=conj(w.'*Q);
if algo.cumstar(1)==1
    w=wc;
end

   




