function [W,crit] = GradientKurtC(epsilon, Nmax, y,f_ini,choix_mu,varargin);

% [W,crit] = GradientKurtC(epsilon,Nmax,y,f_ini,choix_mu,varargin);
%
% Gradient optimization of a kurtosis based contrast function (complex 
% valued signals)
% GradientKurtC performs a gradient ascent optimization of the contrast
% given by the function CostKurtC
%
% Inputs:
% -------
% epsilon = gradient stopping criterion (minimum update difference)
% Nmax = maximum number of gradient iterations
% y = (Nx x Nsample) matrix containing the Nx observations of the mixture
% f_ini = (1 x D*Nx) vector: initial value for the gradient search of the
%   separating filter
% choix_mu : determines the update strategy
%   choix_mu = 'optim' : optimal step size (not yet implemented)
%   choix_mu = 'adapt' : adaptive step size. The step size is multiplied by
%       1/2 each time the criterion does not increase.
%   choix_mu = numerical value: fixed step size set to the given value
% varargin : if choix_mu = 'adapt', varargin = initial step size value (set
%   to 1 by default)
%
% Outputs: 
% -------
% W = (1 x D*Nx) vector: separating filter
% crit = final value of the contrast
%
% -------
% Author: Sebastien Houcke / Marc Castella
% Created: ??/2001
% Last modified: 04/07/2006
% Contact: marc.castella@int-evry.fr

% Autres rq:
%% [W,crit] = simon_grad(epsilon,Nmax,y,f_ini,choix_mu,varargin)
%%
%% Algorithme de Maximisation de (Kurtosis)^2
%% (implementation du gradient)
%%
%% epsilon = precision de calcul sur dJ/df
%% Nmax = nombre maximum d'iterations
%% y = signal d'entree (qxT).
%% f_ini = vecteur d'initialisation de la recherche par gradient
%% choix_mu = pas du gradient: constant / 'adapt' / 'optim'
%% varargin = valeur du pas au depart si 'adapt'
%%
%% f = solution de l'algorithme (1xq*N)
%% crit = valeur finale du critere
%%
%% Site   : Universite de Marne la Vallee
%% Auteur : Houcke <houcke@univ-mlv.fr>
%% Date   : 05/09/01
%% modifie 09/05/2003 Marc Castella
%% modifie 22/05/2003 Marc Castella : ajout de varargin
%% modifie 25/01/2005 Marc Castella : mise a jour de l'aide 

% Initialisation de diverses matrices
Y = [];
ndelay = length(f_ini)/size(y,1);
for ii = 1:ndelay,
    %Y = [ Y; y(:,ii:end+ii-ndelay)];
    Y = [ y(:,ii:end+ii-ndelay); Y];
end;
R = 1/length(Y)*Y*Y'; %%%R = cov(Y');
f = f_ini'; %%% modif MC 09/05/2003

% Algo du gradient
Ni = 0;
df = 1;
if isnumeric(choix_mu),
    mu = -abs(choix_mu);
else,
    if nargin==6,
        mu = -abs(varargin{1});
    else,
        mu = -1;
    end;
end;

%%OLD%while ((sqrt(sum(df.*conj(df))/length(df)) > epsilon) & (Ni < Nmax))  
delta_f = 1;
while ((norm(delta_f)/length(df) > epsilon) & (Ni < Nmax)) | (Ni <50),

    
    Ni = Ni + 1;
    
    [df, dfb,crit_tmp] = DevKurtC(Y,f);

    crit(Ni) = crit_tmp;%%%
    
    if choix_mu == 'optim',
        mu = OptStepC(Y, R, f, dfb);
        newf = f - mu*dfb;
        newf = newf/sqrt(newf'*R*newf);
    elseif choix_mu == 'adapt',
        newf = f - mu*dfb;
        newf = newf/sqrt(newf'*R*newf);
        if crit_tmp>CostKurtC(Y,newf),
            mu = mu/2;
            newf = f-mu*dfb;
            newf = newf/sqrt(newf'*R*newf);
        end;
    elseif isnumeric(choix_mu),
        % mu = -choix_mu;
        newf = f - mu*dfb;
        newf = newf/sqrt(newf'*R*newf);
    end;
 
    delta_f = f-newf;
    f = newf;
end;

%%%crit = simon_cost(Y,f);
f = f'; % f est maintenant un vecteur ligne
W = reshape(f,1,size(y,1),ndelay); %W est sous format standard tableau 1 x Nobserv x D


%plot(crit); disp(choix_mu); disp(mu); pause;